"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CLIOptions = void 0;
const tslib_1 = require("tslib");
const FS = tslib_1.__importStar(require("fs"));
const OS = tslib_1.__importStar(require("os"));
const Path = tslib_1.__importStar(require("path"));
const clime_1 = require("clime");
const prompts_1 = tslib_1.__importDefault(require("prompts"));
const tiva_1 = require("tiva");
const main_1 = require("../main");
const shared_1 = require("../shared");
const JSON_CONFIG_INDENTATION = 2;
const DIR_DEFAULT = Path.resolve(OS.homedir(), '.makescript', 'agent');
const CONFIG_FILE_NAME = 'makescript-agent.json';
class CLIOptions extends clime_1.Options {
}
tslib_1.__decorate([
    clime_1.option({
        flag: 's',
        description: 'MakeScript server URL with token.',
        type: String,
    }),
    tslib_1.__metadata("design:type", Object)
], CLIOptions.prototype, "serverURL", void 0);
tslib_1.__decorate([
    clime_1.option({
        flag: 'd',
        description: 'Directory containing MakeScript node config file and contents.',
        default: DIR_DEFAULT,
    }),
    tslib_1.__metadata("design:type", clime_1.Castable.Directory)
], CLIOptions.prototype, "dir", void 0);
exports.CLIOptions = CLIOptions;
let default_1 = class default_1 extends clime_1.Command {
    async execute({ dir, serverURL }) {
        let configFilePath = Path.join(dir.fullName, CONFIG_FILE_NAME);
        let configFileExists = FS.existsSync(configFilePath);
        if (!configFileExists) {
            let dirname = dir.fullName;
            if (!FS.existsSync(dirname)) {
                FS.mkdirSync(dirname, { recursive: true });
            }
            let answer = await initialQuestions();
            let jsonConfig = {
                name: answer.name,
                server: {
                    url: answer.serverURL,
                },
                scripts: {
                    git: answer.repoURL,
                    // if subPath is "", pass undefined instead.
                    dir: answer.subPath || undefined,
                },
                proxy: undefined,
            };
            writeConfig(jsonConfig);
        }
        let jsonConfig = readConfig();
        if (serverURL && jsonConfig.server.url !== serverURL) {
            jsonConfig.server.url = serverURL;
            writeConfig(jsonConfig);
        }
        let tiva = new tiva_1.Tiva({
            project: Path.join(__dirname, '../../../src/program'),
        });
        shared_1.logger.info('Checking config file ...');
        try {
            await tiva.validate({
                module: './config',
                type: 'JSONConfigFile',
            }, jsonConfig);
            await main_1.main(tiva, {
                ...jsonConfig,
                workspace: dir.fullName,
                agentModule: undefined,
            });
        }
        catch (error) {
            if (error.diagnostics) {
                shared_1.logger.error(`Config file structure does not match:\n${error.diagnostics}`);
            }
            throw error;
        }
        function writeConfig(config) {
            let jsonConfigText = JSON.stringify(config, undefined, JSON_CONFIG_INDENTATION);
            FS.writeFileSync(configFilePath, jsonConfigText);
        }
        function readConfig() {
            let jsonConfigText = FS.readFileSync(configFilePath).toString();
            return JSON.parse(jsonConfigText);
        }
        async function initialQuestions() {
            var _a;
            let promptObjects = [];
            if (!serverURL) {
                promptObjects.push({
                    type: 'text',
                    name: 'serverURL',
                    message: 'Enter MakeScript server URL with token',
                    validate: value => /^https?:\/\/.+$/.test(value),
                });
            }
            promptObjects.push(...[
                {
                    type: 'text',
                    name: 'name',
                    message: 'Enter the name to register as',
                },
                {
                    type: 'text',
                    name: 'repoURL',
                    message: 'Enter the scripts repo url',
                    validate: value => /^(https?:\/\/.+)|(\w+\.git)$/.test(value),
                },
                {
                    type: 'text',
                    name: 'subPath',
                    message: 'Enter the scripts definition dir path',
                },
            ]);
            let answer = await prompts_1.default(promptObjects);
            // There is a bug (or unhandled behavior) with 'prompts'.
            // When user press CTRL + C , program will continue to execute with empty answers.
            // https://github.com/terkelg/prompts/issues/252
            if ((!serverURL && !answer.serverURL) ||
                !answer.name ||
                !answer.repoURL) {
                process.exit(0);
            }
            return {
                serverURL: serverURL !== null && serverURL !== void 0 ? serverURL : answer.serverURL,
                name: answer.name,
                repoURL: answer.repoURL,
                subPath: (_a = answer.subPath) !== null && _a !== void 0 ? _a : undefined,
            };
        }
    }
};
tslib_1.__decorate([
    clime_1.metadata,
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [CLIOptions]),
    tslib_1.__metadata("design:returntype", Promise)
], default_1.prototype, "execute", null);
default_1 = tslib_1.__decorate([
    clime_1.command()
], default_1);
exports.default = default_1;
//# sourceMappingURL=default.js.map